@@ -16,7 +16,7 @@
 ***********************************************************************/
 
 static char const RCSID[] =
-"$Id: mimedefang.c 14671 2007-04-02 17:33:57Z dfs $";
+"$Id$";
 
 #define _BSD_SOURCE 1
 
@@ -1444,6 +1444,8 @@ eom(SMFICTX *ctx)
     char chunk[CHUNK];
     char *hdr, *val, *count;
     char *code, *dsn, *reply;
+    char *postfix_queueid;
+
     struct stat statbuf;
     dynamic_buffer dbuf;
     struct timeval start, finish;
@@ -1462,6 +1464,25 @@ eom(SMFICTX *ctx)
     }
 
     dbuf_init(&dbuf);
+
+    /* Postfix doesn't give us a queue-ID until after first
+       RCPT.  If we don't have a queue-ID, see if we can
+       get one now */
+    if (data->qid == NOQUEUE || !(data->qid)) {
+	postfix_queueid = smfi_getsymval(ctx, "i");
+	if (postfix_queueid && *postfix_queueid) {
+	    data->qid = strdup_with_log(postfix_queueid);
+	    if (!data->qid) {
+		dbuf_free(&dbuf);
+		data->qid = NOQUEUE;
+		cleanup(ctx);
+		DEBUG_EXIT("eom", "SMFIS_TEMPFAIL");
+		return SMFIS_TEMPFAIL;
+	    }
+	    append_mx_command(&dbuf, 'Q', postfix_queueid);
+	}
+    }
+
     /* Signal suspicious body chars */
     if (data->suspiciousBody) {
 	append_mx_command(&dbuf, '?', NULL);
@@ -1472,10 +1493,12 @@ eom(SMFICTX *ctx)
 
     data->cmdFD = get_fd(data, "COMMANDS", data->cmdFD);
     if (data->cmdFD < 0) {
+	dbuf_free(&dbuf);
 	cleanup(ctx);
 	DEBUG_EXIT("eom", "SMFIS_TEMPFAIL");
 	return SMFIS_TEMPFAIL;
     }
+
     if (write_dbuf(&dbuf, data->cmdFD, data, "COMMANDS") < 0) {
 	dbuf_free(&dbuf);
 	cleanup(ctx);
